<?php

/*
Copyright 2008 iThemes (email: support@ithemes.com)

Written by Chris Jean
Version 1.0.2

Version History
	1.0.1 - 2008-09-30
	1.0.2 - 2008-10-03
	1.0.3 - 2008-11-02
		Fixed file path to url conversion issues
*/


if ( !class_exists( 'iThemesFileUtility' ) ) {
	class iThemesFileUtility {
		// Customized media_handle_upload() from 2.6.2 wp-admin/includes/media.php
		function uploadFile( $file_id ) {
			$overrides = array( 'test_form' => false );
			$file = wp_handle_upload( $_FILES[$file_id], $overrides );
			
			if ( isset( $file['error'] ) )
				return new WP_Error( 'upload_error', $file['error'] );
			
			$url = $file['url'];
			$type = $file['type'];
			$file = $file['file'];
			$title = preg_replace( '/\.[^.]+$/', '', basename( $file ) );
			$content = '';
			
			// use image exif/iptc data for title and caption defaults if possible
			if ( $image_meta = @wp_read_image_metadata( $file ) ) {
				if ( trim( $image_meta['title'] ) )
					$title = $image_meta['title'];
				if ( trim( $image_meta['caption'] ) )
					$content = $image_meta['caption'];
			}
			
			// Construct the attachment array
			$attachment = array(
				'post_mime_type' => $type,
				'guid' => $url,
				'post_title' => $title,
				'post_content' => $content
			);
			
			// Save the data
			$id = wp_insert_attachment( $attachment, $file );
			if ( !is_wp_error( $id ) ) {
				wp_update_attachment_metadata( $id, wp_generate_attachment_metadata( $id, $file ) );
			}
			
			
			$data = array(
				'id'		=> $id,
				'url'		=> $url,
				'type'		=> $type,
				'file'		=> $file,
				'title'		=> $title,
				'caption'	=> $content
			);
			
			return $data;
		}
		
		// Wrapper for image_resize
		// Created to provide better return variables
		function resizeImage( $file, $max_w, $max_h, $crop = false, $suffix = null, $dest_path = null, $jpeg_quality = 90 ) {
			$uploads = wp_upload_dir();
			
			if ( empty( $dest_path ) )
				$dest_path = iThemesFileUtility::_getResizedImageFilePath( $file, $max_w, $max_h, $crop, $suffix, $dest_path, $jpeg_quality );
			
			
			
			$this->_pluginPath = dirname( __FILE__ );
			
			$abspath = ABSPATH;
			
			if ( preg_match( '/^[a-zA-Z]:/', $abspath ) )
				$abspath = preg_replace( '|\/$|', '\\', $abspath );
			
			$this->_pluginRelativePath = str_replace( $abspath, '', $this->_pluginPath );
			
			$this->_pluginURL = get_option( 'siteurl' ) . '/' . str_replace( '\\', '/', $this->_pluginRelativePath );
			
			
			
			
			if ( file_exists( $dest_path ) ) {
				if ( filemtime( $file ) > filemtime( $dest_path ) )
					unlink( $dest_path );
				else {
					$abspath = ABSPATH;
					
					if ( preg_match( '/^[a-zA-Z]:/', $abspath ) )
						$abspath = preg_replace( '|\/$|', '\\', $abspath );
					
					$url = get_option( 'siteurl' ) . '/' . str_replace( '\\', '/', str_replace( $abspath, '', $dest_path ) );
					
					
					return array( 'file' => $dest_path, 'url' => $url );
				}
			}
			
			
			$image = image_resize( $file, $max_w, $max_h, $crop, $suffix, $dest_path, $jpeg_quality );
			
			if ( is_wp_error( $image ) )
				return $image;
			
			$url = $uploads['url'] . '/' . basename( $image );
			
			return array( 'file' => $image, 'url' => $url );
		}
		
		function _getResizedImageFilePath( $file, $max_w, $max_h, $crop = false, $suffix = null, $dest_path = null, $jpeg_quality = 90 ) {
			require_once( ABSPATH . 'wp-admin/includes/image.php' );
			
			$image = wp_load_image( $file );
			if ( !is_resource( $image ) )
				return new WP_Error( 'error_loading_image', $image );
			
			list( $orig_w, $orig_h, $orig_type ) = getimagesize( $file );
			$dims = image_resize_dimensions( $orig_w, $orig_h, $max_w, $max_h, $crop );
			if ( ! $dims )
				return $file;
			list( $dst_x, $dst_y, $src_x, $src_y, $dst_w, $dst_h, $src_w, $src_h ) = $dims;
			
			$newimage = imagecreatetruecolor( $dst_w, $dst_h);
			
			// preserve PNG transparency
			if ( IMAGETYPE_PNG == $orig_type && function_exists( 'imagealphablending' ) && function_exists( 'imagesavealpha' ) ) {
				imagealphablending( $newimage, false);
				imagesavealpha( $newimage, true);
			}
			
			imagecopyresampled( $newimage, $image, $dst_x, $dst_y, $src_x, $src_y, $dst_w, $dst_h, $src_w, $src_h);
			
			// we don't need the original in memory anymore
			imagedestroy( $image );
			
			// $suffix will be appended to the destination filename, just before the extension
			if ( !$suffix )
				$suffix = "{$dst_w}x{$dst_h}";
			
			$info = pathinfo($file);
			$dir = $info['dirname'];
			$ext = $info['extension'];
			$name = basename($file, ".{$ext}");
			if ( !is_null($dest_path) and $_dest_path = realpath($dest_path) )
				$dir = $_dest_path;
			$destfilename = "{$dir}/{$name}-{$suffix}.{$ext}";
			
			return $destfilename;
		}
	}
}

?>