<?php

/*
Copyright 2008 iThemes (email: support@ithemes.com)

Written by Chris Jean
Version 1.0.5

Version History
	1.0.1 - 2008-09-16
	1.0.2 - 2008-10-02
	1.0.3 - 2008-10-06
	1.0.4 - 2008-10-07
	1.0.5 - 2008-10-31
		Patched code to handle edge case conditions on some Windows servers
*/


if ( ! class_exists( 'iThemesFeaturedImages' ) ) {
	class iThemesFeaturedImages {
		var $var = 'ithemes_featured_images';
		var $name = 'iThemesFeaturedImages';
		var $version = '1.0.5';
		
		var $idName = 'featured-image-fade';
		var $width = 100;
		var $height = 100;
		var $defaultSleep = 2;
		var $defaultFade = 1;
		
		var $_class = '';
		var $_initialized = false;
		var $_options = array();
		
		var $_userID = 0;
		var $_usedInputs = array();
		var $_selectedVars = array();
		var $_pluginPath = '';
		var $_pluginRelativePath = '';
		var $_pluginURL = '';
		
		
		function iThemesFeaturedImages( $idName, $width, $height, $sleep = false, $fade = false ) {
			$this->idName = $idName;
			$this->width = $width;
			$this->height = $height;
			
			if ( false !== $sleep )
				$this->defaultSleep = $sleep;
			if ( false !== $fade )
				$this->defaultFade = $fade;
			
			add_action( 'init', array( &$this, 'init' ) );
			add_action( 'wp_head', array( &$this, 'initImages' ) );
			add_action( 'wp_print_scripts', array( &$this, 'addScripts' ) );
		}
		
		function init() {
			if( ! $this->_initialized ) {
				$this->_setVars();
				$this->_load();
				
				load_plugin_textdomain( $this->var, $this->_pluginRelativePath . '/lang' );
				
				$this->_initialized = true;
			}
		}
		
		function addScripts() {
			if ( is_front_page() ) {
				wp_enqueue_script( 'jquery' );
				wp_enqueue_script( 'jquery-cross-slide', get_bloginfo( 'stylesheet_directory' ) . '/js/jquery.cross-slide.js' );
			}
		}
		
		function initImages() {
			if ( is_front_page() )
				$this->_fadeImages();
		}
		
		function _setVars() {
			$this->_class = get_class( $this );
			
			$user = wp_get_current_user();
			$this->_userID = $user->ID;
			
			
			if ( !defined( 'WP_CONTENT_URL' ) )
				define( 'WP_CONTENT_URL', get_option( 'siteurl' ) . '/wp-content');
			if ( !defined( 'WP_CONTENT_DIR' ) )
				define( 'WP_CONTENT_DIR', ABSPATH . 'wp-content' );
			
			$this->_pluginPath = dirname( __FILE__ );
			
			$abspath = ABSPATH;
			
			if ( preg_match( '/^[a-zA-Z]:/', $abspath ) )
				$abspath = preg_replace( '|\/$|', '\\', $abspath );
			
			$this->_pluginRelativePath = str_replace( $abspath, '', $this->_pluginPath );
			
			$this->_pluginURL = get_option( 'siteurl' ) . '/' . str_replace( '\\', '/', $this->_pluginRelativePath );
		}
		
		
		// Options Storage ////////////////////////////
		
		function _initializeOptions() {
			$this->_options['imageIDs'] = array();
			$this->_options['fadeSort'] = 'ordered';
			$this->_options['enableFade'] = 1;
			$this->_options['sleep'] = $this->defaultSleep;
			$this->_options['fade'] = $this->defaultFade;
			
			$this->_save();
		}
		
		function _save() {
			$data = $this->_options;
			
			if ( $data == @get_option( $this->var ) )
				return true;
			
			return update_option( $this->var, $data );
		}
		
		function _load() {
			$data = @get_option( $this->var );
			
			if ( is_array( $data ) ) {
				$this->_options = $data;
				
				
				$this->_options['sleep'] = floatval( $this->_options['sleep'] );
				$this->_options['fade'] = floatval( $this->_options['fade'] );
				
				if ( $this->_options['sleep'] <= 0 )
					$this->_options['sleep'] = $this->defaultSleep;
				if ( $this->_options['fade'] <= 0 )
					$this->_options['fade'] = $this->defaultFade;
				if ( empty( $this->_options['fadeSort'] ) )
					$this->_options['fadeSort'] = 'ordered';
			}
			else {
				$this->_initializeOptions();
				$this->_initializeImages();
			}
		}
		
		
		// Pages //////////////////////////////////////
		
		function editPage() {
			global $wpdb;
			
			if ( function_exists( 'current_user_can' ) && ! current_user_can( 'edit_themes' ) )
				die( __( 'Cheatin uh?' ) );
			
			
			if ( count( $this->_options ) < 1 )
				$this->_load();
			
			
			if ( ! empty( $_POST['upload'] ) ) {
				if ( is_array( $_FILES['imageFile'] ) && ( 0 === $_FILES['imageFile']['error'] ) ) {
					require_once( 'iThemesFileUtility.php' );
					
					check_admin_referer( $this->var . '-nonce' );
					
					$file = iThemesFileUtility::uploadFile( 'imageFile' );
					
					if ( is_wp_error( $file ) )
						$this->_showErrorMessage( 'Unable to save uploaded image. Ensure that the web server has permissions to write to the uploads folder' );
					else {
						$this->_options['imageIDs'][] = $file['id'];
						$this->_save();
					}
				}
				else
					$this->_showErrorMessage( 'You must add a file by clicking the browse button first.' );
			}
			elseif ( ! empty( $_GET['delete'] ) ) {
				wp_delete_attachment( $_GET['delete'] );
				
				$ids = array();
				
				foreach ( (array) $this->_options['imageIDs'] as $id )
					if ( $id != $_GET['delete'] )
						$ids[] = $id;
				
				
				$this->_options['imageIDs'] = $ids;
				$this->_save();
			}
			elseif ( ! empty( $_POST['saveSettings'] ) ) {
				check_admin_referer( $this->var . '-nonce' );
				
				if ( ! empty( $_POST['used-inputs'] ) ) {
					$usedInputs = explode( ',', $_POST['used-inputs'] );
					
					foreach ( (array) $usedInputs as $input ) {
						if ( isset( $_POST[$this->var][$input] ) )
							$this->_options[$input] = $_POST[$this->var][$input];
						elseif ( isset( $this->_options[$input] ) )
							unset( $this->_options[$input] );
					}
				}
				else
					foreach ( $_POST[$this->var] as $key => $val )
						$this->_options[$key] = $val;
				
				
				$errorCount = 0;
				
				if ( ( $this->_options['sleep'] != floatval( $this->_options['sleep'] ) ) || ( floatval( $this->_options['sleep'] ) <= 0 ) )
					$errorCount++;
				if ( ( $this->_options['fade'] != floatval( $this->_options['fade'] ) ) || ( floatval( $this->_options['fade'] ) <= 0 ) )
					$errorCount++;
				
				if ( $errorCount < 1 ) {
					if ( $this->_options['sleep'] <= 0 )
						$this->_options['sleep'] = 2;
					if ( $this->_options['fade'] <= 0 )
						$this->_options['fade'] = 1;
					if ( empty( $this->_options['fadeSort'] ) )
						$this->_options['fadeSort'] = 'ordered';
					
					if ( $this->_save() )
						$this->_showStatusMessage( __( 'Settings updated', $this->var ) );
					else
						$this->_showErrorMessage( __( 'Error while updating settings', $this->var ) );
				}
				else {
					$this->_showErrorMessage( __( 'The fade options timing values must be numeric values greater than 0.', $this->var ) );
					
					$this->_showErrorMessage( __ngettext( 'Please fix the input marked in red below.', 'Please fix the inputs marked in red below.', $errorCount ) );
				}
			}
			
?>
	<div class="wrap">
		<h2><?php _e( 'Images', $this->var ); ?></h2>
		<br />
		<div>The uploaded image should be <?php echo $this->width . 'x' . $this->height; ?> (<?php echo $this->width; ?>px wide by <?php echo $this->height; ?>px high).</div>
		
		<table class="form-table">
<?php
			
			$files = array();
			
			foreach ( (array) $this->_options['imageIDs'] as $id ) {
				if ( wp_attachment_is_image( $id ) ) {
					$post = get_post( $id );
					
					$file = array();
					$file['ID'] = $id;
					$file['file'] = get_attached_file( $id );
					$file['url'] = wp_get_attachment_url( $id );
					$file['title'] = $post->post_title;
					$file['name'] = basename( get_attached_file( $id ) );
					
					$files[] = $file;
				}
			}
			
			usort( $files, array( &$this, _sortFiles ) );
			
			
			require_once( 'iThemesFileUtility.php' );
			
?>
			<?php foreach ( (array) $files as $file ) : ?>
				<?php $thumb = iThemesFileUtility::resizeImage( $file['file'], 100, 100 ); ?>
				<tr>
					<th scope="row">
						<a href="<?php echo $file['url']; ?>" target="imagePreview">
							<?php if ( ! is_wp_error( $thumb ) ) : ?>
								<img src="<?php echo $thumb['url']; ?>" alt="<?php echo $file; ?>" />
							<?php else : ?>
								Thumbnail generation error
								<!-- Theme Error: <?php echo $thumb->get_error_message(); ?> -->
							<?php endif; ?>
						</a>
					</th>
					<td style="vertical-align:top;">
						<?php echo $file['name']; ?><br />
						<a href="<?php echo $this->_getBackLink(); ?>&amp;delete=<?php echo urlencode( $file['ID'] ); ?>" onclick="if(!confirm('Are you sure you want to delete <?php echo $file['name']; ?>?')) return false;">Delete</a>
						<br />
					</td>
				</tr>
			<?php endforeach; ?>
			
			<tr>
				<th scope="row">Add a new image</th>
				<td>
					<form enctype="multipart/form-data" method="post" action="<?php echo $this->_getBackLink(); ?>">
						<?php echo wp_nonce_field( $this->var . '-nonce' ); ?>
						Select a file: <input type="file" name="imageFile" /><br /><br />
						<input type="submit" name="upload" value="Upload" />
					</form>
				</td>
			</tr>
		</table>
		<br /><br />
		
		<h2><?php _e( 'Settings', $this->var ); ?></h2>
		
		<form enctype="multipart/form-data" method="post" action="<?php echo $this->_getBackLink(); ?>">
			<table class="form-table">
				<tr>
					<th scope="row">Fade Animation</th>
					<td>
						<div>The fade animation will show each of the images with a smooth fade transition between each image.</div>
						<div>If the animation is disabled, a single random image will be shown.</div>
						<br />
						
						<?php $this->_addCheckBox( $this->var, 'enableFade', '1', 'Enable' ); ?>
					</td>
				</tr>
				<tr>
					<th scope="row">Fade Options</th>
					<td>
						<div>The following options control the fade animation.</div>
						<div>If the animation is disabled, these options will not make any effect.</div>
						<br />
						
						<?php $this->_addRadio( $this->var, 'fadeSort', 'ordered', 'Order the images by alphabetically by file name.' ); ?><br />
						<?php $this->_addRadio( $this->var, 'fadeSort', 'random', 'Order the images randomly.' ); ?><br />
						<br />
						
						<table>
							<tr>
								<td style="margin:0px; border-bottom-width:0px; line-height:10px; padding:0px 10px 0px 0px;">
									Length of time to display each image in seconds
								</td>
								<?php if ( ( ! empty( $_POST['saveSettings'] ) ) && ( floatval( $_POST[$this->var]['sleep'] ) <= 0 ) ) : ?>
									<td style="margin:0px; border-bottom-width:0px; line-height:10px; text-align:center; padding:3px; background-color:red;">
								<?php else: ?>
									<td style="margin:0px; border-bottom-width:0px; line-height:10px; text-align:center; padding:0px;">
								<?php endif; ?>
									<input type="text" name="<?php echo $this->var; ?>[sleep]" value="<?php if ( ! empty( $_POST['saveSettings'] ) ) : echo $_POST[$this->var]['sleep']; else : echo $this->_options['sleep'];  endif; ?>" size="3" maxlength="5" />
									<?php $this->_usedInputs[] = 'sleep'; ?>
								</td>
							</tr>
							<tr>
								<td style="margin:0px; border-bottom-width:0px; line-height:10px; padding:0px 10px 0px 0px;">
									Length of time to fade each image in seconds
								</td>
								<?php if ( ( ! empty( $_POST['saveSettings'] ) ) && ( floatval( $_POST[$this->var]['fade'] ) <= 0 ) ) : ?>
									<td style="margin:0px; border-bottom-width:0px; line-height:10px; text-align:center; padding:3px; background-color:red;">
								<?php else: ?>
									<td style="margin:0px; border-bottom-width:0px; line-height:10px; text-align:center; padding:0px;">
								<?php endif; ?>
									<input type="text" name="<?php echo $this->var; ?>[fade]" value="<?php echo $this->_options['fade']; ?>" size="3" maxlength="5" />
									<?php $this->_usedInputs[] = 'fade'; ?>
								</td>
							</tr>
						</table>
					</td>
				</tr>
			</table>
			
			<?php $this->_addSubmitButton( 'saveSettings', 'Save Settings' ); ?>
			<?php $this->_addUsedInputs(); ?>
			<?php wp_nonce_field( $this->var . '-nonce' ); ?>
		</form>
	</div>
<?php
		}
		
		
		// Plugin Functions ///////////////////////////
		
		function _addSubmitButton( $name, $value, $disabled = false ) {
			echo '<p class="submit"><input type="submit" name="' . $name . '" value="' . $value . '" /></p>';
		}
		
		function _addRadio( $base, $variable, $value, $message, $disabled = false ) {
			$selected = '';
			
			if ( isset( $_POST[$base][$variable] ) ) {
				if ( $value == $_POST[$base][$variable] )
					$selected = ' checked="checked"';
			}
			elseif ( empty( $this->_selectedVars[$variable] ) ) {
				$this->_selectedVars[$variable] = true;
				$selected = ' checked="checked"';
			}
			
?>
						<label for="<?php echo $base ?>-<?php echo $variable; ?>-<?php echo $value; ?>">
							<input name="<?php echo $base ?>[<?php echo $variable; ?>]" type="radio" id="<?php echo $base ?>-<?php echo $variable; ?>-<?php echo $value; ?>" value="<?php echo $value; ?>"<?php echo $selected ?><?php if ( $disabled ) echo ' disabled'; ?> />
							<?php echo $message; ?>
						</label>
<?php
			
			$this->_usedInputs[] = $variable;
		}
		
		function _addCheckBox( $base, $variable, $value, $message, $disabled = false, $selected = false ) {
			if ( $this->_options[$variable] == $value )
				$selected = true;
			
?>
						<label for="<?php echo $base ?>-<?php echo $variable; ?>">
							<input name="<?php echo $base ?>[<?php echo $variable; ?>]" type="checkbox" id="<?php echo $base ?>-<?php echo $variable; ?>" value="<?php echo $value; ?>"<?php if ( ( ! empty( $_POST[$base][$variable] ) && ( $value == $_POST[$base][$variable] ) ) || $selected ) echo ' checked="checked"'; ?><?php if ( $disabled ) echo ' disabled'; ?> />
							<?php echo $message; ?>
						</label>
<?php
			
			$this->_usedInputs[] = $variable;
		}
		
		function _addUsedInputs() {
			$usedInputs = '';
			
			$used = array();
			
			foreach ( (array) $this->_usedInputs as $input ) {
				if ( empty( $used[$input] ) ) {
					if ( ! empty( $usedInputs ) )
						$usedInputs .= ',';
					
					$used[$input] = 1;
					$usedInputs .= $input;
				}
			}
			
			if ( ! empty( $usedInputs ) )
				echo '<input type="hidden" name="used-inputs" value="' . $usedInputs . '" />' . "\n";
		}
		
		function _fadeImages() {
			require_once( 'iThemesFileUtility.php' );
			
			
			$files = array();
			
			foreach ( (array) $this->_options['imageIDs'] as $id ) {
				if ( wp_attachment_is_image( $id ) ) {
					$post = get_post( $id );
					
					$file = get_attached_file( $id );
					$data = iThemesFileUtility::resizeImage( $file, $this->width, $this->height, true );
					
					if ( ! is_array( $data ) && is_wp_error( $data ) )
						echo "<!-- Resize Error: " . $data->get_error_message() . " -->";
					else
						$files[] = $data['url'];
				}
			}
			
			if ( 0 === count( $files ) ) {
				if ( $dir = @opendir( TEMPLATEPATH . '/images/random/' ) ) {
					while ( ( $file = readdir( $dir ) ) !== false )
						if ( is_file( TEMPLATEPATH . '/images/random/' . $file ) && ( preg_match( '/gif$|jpg$|jpeg$|png$/i', $file ) ) )
							$files[] = get_bloginfo( 'template_directory' ) . '/images/random/' . $file;
					
					closedir( $dir );
				}
			}
			
			if ( ( '1' == $this->_options['enableFade'] ) && ( count( $files ) > 1 ) ) {
				if ( 'ordered' == $this->_options['fadeSort'] )
					usort( $files, array( &$this, _sortFiles ) );
				else
					shuffle( $files );
				
				$list = '';
				
				foreach ( (array) $files as $file ) {
					if ( ! empty( $list ) )
						$list .= ',';
					
					$list .= "{src: '$file'}";
				}
				
?>
	<style type="text/css">
		#<?php echo $this->idName; ?> {
			width: <?php echo $this->width; ?>px;
			height: <?php echo $this->height; ?>px;
		}
	</style>
	
	<script type='text/javascript'>
		jQuery(function($) {$(document).ready(function() {
			$('#<?php echo $this->idName; ?>').crossSlide(
				{sleep: <?php echo $this->_options['sleep']; ?>, fade: <?php echo $this->_options['fade']; ?>},
				[<?php echo $list; ?>]
			);
		});});
	</script>
<?
				
			}
			else {
				shuffle( $files );
				
?>
	<style type="text/css">
		#<?php echo $this->idName; ?> {
			width: <?php echo $this->width; ?>px;
			height: <?php echo $this->height; ?>px;
			background-image: url( <?php echo $files[0]; ?> );
			background-repeat: no-repeat;
		}
	</style>
<?php
				
			}
		}
		
		function _sortFiles( $a, $b ) {
			if ( is_array( $a ) ) {
				$a = basename( $a['name'] );
				$b = basename( $b['name'] );
			}
			else {
				$a = preg_replace( '/-\d+x\d+\./', '.', $a );
				$b = preg_replace( '/-\d+x\d+\./', '.', $b );
				
				$a = basename( $a );
				$b = basename( $b );
			}
			
			return strnatcasecmp( $a, $b );
		}
		
		function _getBackLink() {
			return $_SERVER['REQUEST_URI'];
		}
		
		function _showStatusMessage( $message ) {
			
?>
	<div id="message" class="updated fade"><p><strong><?php echo $message; ?></strong></p></div>
<?php
			
		}
		
		function _showErrorMessage( $message ) {
			
?>
	<div id="message" class="error"><p><strong><?php echo $message; ?></strong></p></div>
<?php
			
		}
		
		
		// Utility Functions //////////////////////////
		
		function _initializeImages() {
			if ( $dir = @opendir( TEMPLATEPATH . '/images/random/' ) ) {
				require_once( 'iThemesFileUtility.php' );
				
				if ( ! ( ( $uploads = wp_upload_dir() ) && false === $uploads['error'] ) )
					return new WP_Error( 'upload_dir_failure', 'Unable to load images into the uploads directory: ' . $uploads['error'] );
				
				
				$ids;
				
				while ( ( $file = readdir( $dir ) ) !== false ) {
					if ( is_file( TEMPLATEPATH . '/images/random/' . $file ) && ( preg_match( '/gif$|jpg$|jpeg$|png$/i', $file ) ) ) {
						$filename = wp_unique_filename( $uploads['path'], basename( $file ) );
						
						// Move the file to the uploads dir
						$new_file = $uploads['path'] . "/$filename";
						if ( false === copy( TEMPLATEPATH . '/images/random/' . $file, $new_file ) ) {
							closedir( $dir );
							return new WP_Error( 'copy_file_failure', 'The theme images were unable to be loaded into the uploads directory' );
						}
						
						// Set correct file permissions
						$stat = stat( dirname( $new_file ));
						$perms = $stat['mode'] & 0000666;
						@chmod( $new_file, $perms );
						
						// Compute the URL
						$url = $uploads['url'] . "/$filename";
						
						
						$wp_filetype = wp_check_filetype( $file );
						$type = $wp_filetype['type'];
						
						
						$file_obj['url'] = $url;
						$file_obj['type'] = $type;
						$file_obj['file'] = $new_file;
						
						
						$title = preg_replace( '/\.[^.]+$/', '', basename( $file ) );
						$content = '';
						
						require_once( ABSPATH . 'wp-admin/includes/image.php' );
						
						// use image exif/iptc data for title and caption defaults if possible
						if ( $image_meta = @wp_read_image_metadata( $new_file ) ) {
							if ( trim( $image_meta['title'] ) )
								$title = $image_meta['title'];
							if ( trim( $image_meta['caption'] ) )
								$content = $image_meta['caption'];
						}
						
						// Construct the attachment array
						$attachment = array(
							'post_mime_type' => $type,
							'guid' => $url,
							'post_title' => $title,
							'post_content' => $content
						);
						
						// Save the data
						$id = wp_insert_attachment( $attachment, $new_file );
						if ( ! is_wp_error( $id ) ) {
							wp_update_attachment_metadata( $id, wp_generate_attachment_metadata( $id, $new_file ) );
						}
						
						
						$ids[] = $id;
					}
				}
				
				closedir( $dir );
				
				
				$this->_options['imageIDs'] = $ids;
				$this->_save();
			}
		}
	}
}

?>